"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/lambda/lambda.ts
var lambda_exports = {};
__export(lambda_exports, {
  onEvent: () => onEvent
});
module.exports = __toCommonJS(lambda_exports);

// src/lambda/get-parameters.ts
var import_client_sts = require("@aws-sdk/client-sts");
var import_client_ssm = require("@aws-sdk/client-ssm");
var getSsmClient = async (region, roleArn, sessionName) => {
  if (roleArn && sessionName) {
    const client = new import_client_sts.STSClient({});
    console.log("assuming role", roleArn);
    const res = await client.send(
      new import_client_sts.AssumeRoleCommand({
        RoleArn: roleArn,
        RoleSessionName: sessionName
      })
    );
    const { Credentials } = res;
    if (!Credentials) {
      throw new Error(`no credentials after assuming role ${roleArn}`);
    }
    const { AccessKeyId, Expiration, SecretAccessKey, SessionToken } = Credentials;
    if (!AccessKeyId || !SecretAccessKey) {
      throw new Error(`no AccessKeyId and/or SecretAccessKey after assuming role ${roleArn}`);
    }
    console.log("role assumed", roleArn);
    return new import_client_ssm.SSMClient({
      region,
      credentials: {
        accessKeyId: AccessKeyId,
        secretAccessKey: SecretAccessKey,
        expiration: Expiration,
        sessionToken: SessionToken
      }
    });
  }
  if (roleArn && !sessionName) {
    throw new Error("recieved roleArn but no sessionName");
  }
  console.log("no role, not assuming");
  return new import_client_ssm.SSMClient({
    region
  });
};
var getPaginatedParams = async (ssmClient, path) => {
  console.log("getting params from", path);
  const paginator = (0, import_client_ssm.paginateGetParametersByPath)(
    {
      client: ssmClient
    },
    {
      Path: path,
      Recursive: true,
      WithDecryption: true,
      MaxResults: 10
    }
  );
  const results = [];
  for await (const page of paginator) {
    results.push(...page.Parameters || []);
    console.log("paging", results.length);
  }
  return results;
};
var getParameters = async (region, path, roleArn, sessionName) => {
  const ssmClient = await getSsmClient(region, roleArn, sessionName);
  const params = {};
  const results = await getPaginatedParams(ssmClient, path);
  results.forEach(({ Name, Value }) => {
    if (Name && Value) {
      params[Name] = Value;
    }
  });
  console.log("done");
  return params;
};

// ../../modules/custom-resource-wrapper/src/custom-resource-wrapper.ts
var successEvent = (event, { data }) => {
  const { physicalResourceId, ...rest } = data ?? {};
  const restData = Object.keys(rest).length ? rest : void 0;
  return {
    ...event,
    Status: "SUCCESS",
    Data: restData,
    PhysicalResourceId: physicalResourceId ?? event.PhysicalResourceId ?? event.LogicalResourceId
  };
};
var failureEvent = (event, { reason }) => {
  return {
    ...event,
    Status: "FAILED",
    Reason: reason,
    PhysicalResourceId: event.PhysicalResourceId || event.LogicalResourceId
  };
};
var errToString = (e) => {
  return `[${e.name}] ${e.message}: ${e.stack}`;
};
var customResourceWrapper = (handler) => {
  return async (event) => {
    console.log(JSON.stringify(event));
    const { ServiceToken, ...rp } = event.ResourceProperties;
    const augmentedRP = {
      ...rp,
      requestId: event.RequestId,
      serviceToken: ServiceToken
    };
    try {
      switch (event.RequestType) {
        case "Create": {
          const data = await handler.onCreate(augmentedRP);
          return successEvent(event, { data });
        }
        case "Delete": {
          if (handler.onDelete) {
            await handler.onDelete({
              ...augmentedRP,
              physicalResourceId: event.PhysicalResourceId
            });
          }
          return successEvent(event, {});
        }
        case "Update": {
          if (handler.onUpdate) {
            const data = await handler.onUpdate(
              {
                ...augmentedRP,
                physicalResourceId: event.PhysicalResourceId
              },
              event.OldResourceProperties
            );
            return successEvent(event, { data });
          }
          return successEvent(event, {});
        }
      }
    } catch (e) {
      console.error(e);
      return failureEvent(event, { reason: errToString(e) });
    }
  };
};

// src/lambda/lambda.ts
var onEvent = customResourceWrapper({
  onCreate: ({ stackName, regionName, parameterPath, role }) => getParameters(regionName, parameterPath, role, stackName),
  onUpdate: ({ stackName, regionName, parameterPath, role }) => getParameters(regionName, parameterPath, role, stackName)
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  onEvent
});
