import { CustomResource, aws_iam as iam } from 'aws-cdk-lib';
import { Construct } from 'constructs';
/**
 * Properties of the RemoteParameters
 */
export interface RemoteParametersProps {
    /**
     * The region code of the remote stack.
     */
    readonly region: string;
    /**
     * The assumed role used to get remote parameters.
     */
    readonly role?: iam.IRole;
    /**
     * The parameter path.
     */
    readonly path: string;
    /**
     * Indicate whether always update the custom resource to get the new stack output
     * @default true
     */
    readonly alwaysUpdate?: boolean;
}
/**
 * Represents the RemoteParameters of the remote CDK stack
 */
export declare class RemoteParameters extends Construct {
    /**
     * The parameters in the SSM parameter store for the remote stack.
     */
    readonly parameters: CustomResource;
    constructor(scope: Construct, id: string, props: RemoteParametersProps);
    /**
     * Get the parameter.
     * @param key output key
     */
    get(key: string): string;
}
